// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

#include "fragmentVersionCentroid.h"

#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		#if defined(TEXEL_AA) && defined(TEXEL_AA_FEATURE)
			_centroid in highp vec2 uv0;
			_centroid in highp vec2 uv1;
		#else
			_centroid in vec2 uv0;
			_centroid in vec2 uv1;
		#endif
	#endif
#else
	#ifndef BYPASS_PIXEL_SHADER
		varying vec2 uv0;
		varying vec2 uv1;
	#endif
#endif

in highp vec3 x;

varying vec4 color;
varying vec3 wpos;
varying highp vec3 w_pos;
varying highp vec3 position;
varying highp vec3 p;

#ifdef FOG
varying vec4 fogColor;
#endif

#include "uniformShaderConstants.h"
#include "util.h"
uniform vec2 FOG_CONTROL;
uniform vec4 FOG_COLOR;
uniform float RENDER_DISTANCE;
uniform highp float TIME;
uniform highp float TOTAL_REAL_WORLD_TIME;


#include "Settings.h"

LAYOUT_BINDING(0) uniform sampler2D TEXTURE_0;
LAYOUT_BINDING(1) uniform sampler2D TEXTURE_1;
LAYOUT_BINDING(2) uniform sampler2D TEXTURE_2;

//Player Shadow ^_^
highp float player(highp vec2 uv){
	uv.y *= 1.0;
	uv.y -= 0.2;
	return 1.0 - length(uv);
	}

highp vec3 Tm(highp vec3 x) 
{	
   float A = 1.1;
   float B = 0.;
   float C = 0.4;
   float D = 0.5;
   float E = 0.0;
   float F = 0.7;	
return ((x * (A * x + C * B) + D * E) / (x * (A * x + B) + D * F)) - E / F;
}

//Noise from BSBE
highp float has( highp float n ){
return fract(sin(n)*43758.5453);
}
highp float ray( highp vec2 x ){
	highp vec2 p = floor(x);
	highp vec2 f = fract(x);
    f = f*f*(3.0-2.0*f);

    highp float n = position.x + position.y*57.0;
return mix(mix( has(n+  0.0), has(n+  1.0),f.x), mix( has(n+ 57.0), has(n+ 58.0),f.x),f.y);
}

		
//Water Noise
highp float hash(highp float p) { p = fract(p * 0.011); p *= p + 7.5; p *= p + p; return fract(p); }
highp float hash(highp vec2 p) {highp vec3 p3 = fract(vec3(p.xyx) * 0.13); p3 += dot(p3, p3.yzx + 3.333); return fract((p3.x + p3.y) * p3.z); }

highp float noise(highp float x) {
    highp float i = floor(x);
    highp float f = fract(x);
    highp float u = f * f * (3.0 - 2.0 * f);
    return mix(hash(i), hash(i + 1.0), u);
}

highp float noise(highp vec2 x) {
    highp vec2 i = floor(x);
    highp vec2 f = fract(x);

	// Four corners in 2D of a tile
	highp float a = hash(i);
    highp float b = hash(i + vec2(1.0, 0.0));
    highp float c = hash(i + vec2(0.0, 1.0));
    highp float d = hash(i + vec2(1.0, 1.0));

    // Simple 2D lerp using smoothstep envelope between the values.
	// return vec3(mix(mix(a, b, smoothstep(0.0, 1.0, f.x)),
	//			mix(c, d, smoothstep(0.0, 1.0, f.x)),
	//			smoothstep(0.0, 1.0, f.y)));

	// Same code, with the clamps in smoothstep and common subexpressions
	// optimized away.
    highp vec2 u = f * f * (3.0 - 2.0 * f);
	return mix(a, b, u.x) + (c - a) * u.y * (1.0 - u.x) + (d - b) * u.x * u.y;
}

highp float noise(highp vec3 x) {
    const highp vec3 step = vec3(110, 241, 171);

    highp vec3 i = floor(x);
    highp vec3 f = fract(x);
 
    // For performance, compute the base input to a 1D hash from the integer part of the argument and the 
    // incremental change to the 1D based on the 3D -> 1D wrapping
    highp float n = dot(i, step);

    highp vec3 u = f * f * (3.0 - 2.0 * f);
    return mix(mix(mix( hash(n + dot(step, vec3(0, 0, 0))), hash(n + dot(step, vec3(1, 0, 0))), u.x),
                   mix( hash(n + dot(step, vec3(0, 1, 0))), hash(n + dot(step, vec3(1, 1, 0))), u.x), u.y),
               mix(mix( hash(n + dot(step, vec3(0, 0, 1))), hash(n + dot(step, vec3(1, 0, 1))), u.x),
                   mix( hash(n + dot(step, vec3(0, 1, 1))), hash(n + dot(step, vec3(1, 1, 1))), u.x), u.y), u.z);
}

vec3 saturation(vec3 rgb, float adjustment)
{
    const vec3 W = vec3(0.2125, 0.7154, 0.0721);
    vec3 intensity = vec3(dot(rgb, W));
    return min( mix(intensity, rgb, adjustment), vec3( 1.0, 1.0, 1.0 ));
}

//caustic
#define ray1 length(0.5-fract(k.xyw*=mat3(vec3(-2.0,-1.0,0.0), vec3(3.0,-1.0,1.0), vec3(1.0,-1.0,-1.0))*0.5))
#define ray2 length(0.5-fract(k.xyw*=mat3(vec3(-2.0,-1.0,0.0), vec3(3.0,-1.0,1.0), vec3(1.0,-1.0,-1.0))*0.5))
#define ray3 length(0.5-fract(k.xyw*=mat3(vec3(-2.0,-1.0,0.0), vec3(3.0,-1.0,1.0), vec3(1.0,-1.0,-1.0))*0.5))


#define NUM_NOISE_OCTAVES 10

highp float clouds(highp vec2 x) {
	highp float v = 0.1;
	highp float a = 0.3;
	highp float t = TIME;
	highp vec2 shift = vec2(9);
	// Rotate to reduce axial bias
    mat2 rot = mat2(cos(0.9), sin(0.1), -sin(0.1), -cos(0.9));
	for (int i = 0; i < NUM_NOISE_OCTAVES; ++i) {
		v += a * noise(t*0.02+x);
		x = rot * x * 2.5 + shift;
		a *= 0.7;
	}
	highp float den = smoothstep(0.6,0.5,v);
highp float r = v-(1.0-den);
if(r<0.0)
r = 0.0;
v = 1.-pow(0.2, r);
return v;
}

highp float sunny(highp vec2 uv, highp float blur, highp float sunn){
	highp float sun = length(uv);
highp float suun = smoothstep(sunn, sunn-blur, sun);
return suun;
	}

void main()

{
#ifdef BYPASS_PIXEL_SHADER

	gl_FragColor = vec4(0, 0, 0, 0);
	return;
#else 

#if USE_TEXEL_AA
	vec4 diffuse = texture2D_AA(TEXTURE_0, uv0);
vec4 TD = texture2D(TEXTURE_1, vec2(0,1));
#else
	vec4 diffuse = texture2D(TEXTURE_0, uv0);
vec4 TD = texture2D(TEXTURE_1, vec2(0,1));
#endif


	
#ifdef SEASONS_FAR
	diffuse.a = 1.0;
#endif



#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
	#define ALPHA_THRESHOLD 0.05
	#else
	#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif

//detect time
highp float rainy = smoothstep(0.66,0.3,FOG_CONTROL.x);
highp float dayy = pow(max(min(1.0-FOG_COLOR.r*1.2,1.0),0.0),0.4);
highp float nighty = pow(max(min(1.0-FOG_COLOR.r*1.5,1.0),0.0),1.2);
highp float dusky = clamp(FOG_COLOR.r-FOG_COLOR.g,0.0,0.5)*2.0;

vec4 inColor = color;

float cave = 1.-uv1.y;

//bool
bool nether = (FOG_COLOR.r>FOG_COLOR.b && FOG_COLOR.r<0.5 && FOG_COLOR.b<0.2);
bool underwater = (FOG_COLOR.r<FOG_COLOR.b && FOG_COLOR.b>FOG_COLOR.g && FOG_COLOR.g>FOG_COLOR.r && FOG_CONTROL.x < 0.15);
bool water2 = (uv1.y < 0.9 && abs((2.0 * position.y - 15.0) / 16.0 - uv1.y) < 0.00002);
bool water = (color.b > color.r);
bool isWater = (uv1.y < 0.9 && abs((2.0 * position.y - 15.0) / 16.0 - uv1.y) < 0.00002);

#if defined(BLEND)
	diffuse.a *= inColor.a;
#endif

vec4 deteksi = texture2D(TEXTURE_0,uv0+vec2(.0155,.0));
bool detect_ore= deteksi.r<= .6 && deteksi.g<= .1 && deteksi.b>= .99;


float day = max(0.0, min( 1.0,(TD.r-0.5)*2.0));
float night = 1.0-day;
float sun = 0.5-abs(0.5-day);
highp float sunr = clamp(FOG_COLOR.r-FOG_COLOR.g,0.,.5)*2.;

//night
vec4 diff = texture2D( TEXTURE_1, uv1 );
vec3 nc = vec3(0.8,0.9,1.);
diff.rgb *= pow(diff.rgb, 1.0-nc);

#if !defined(ALWAYS_LIT)
diffuse.rgb = saturation( diffuse.rgb, 1.2 );
if(!(detect_ore))diffuse *= diff;
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = inColor.a;
	#endif
	diffuse.rgb *= mix(inColor.rgb,normalize(inColor.rgb),0.5);
#else
	vec2 uv = inColor.xy;
	diffuse.rgb *= mix(vec3(1.9,2.0,2.2), texture2D( TEXTURE_2, uv).rgb*2.0, inColor.b);
	diffuse.rgb *= inColor.aaa;
	diffuse.a = 1.0;
#endif

#ifdef FANCY
if(color.b+color.b>color.r+color.g||color.g+color.g>color.r+color.g){
	
}
#endif

//color

diffuse.rgb *= vec3(1.8,1.7,1.5);


diffuse.rgb -= 0.09;

//Bloom Light


//Torch
float rtx = smoothstep(0.5,0.8,uv1.x);
//diffuse.rgb += rtx*(vec3(0.5,0.2,0.)*1.)*(uv1.x);
float iii = sin(cos(sin(TOTAL_REAL_WORLD_TIME)));
//vec3 chroma = 0. + 0. * sin(TIME + vec3(4.0, 2., 0.0)*1.);
diffuse.rgb += rtx*iii*(vec3(0.6,0.23,0.02)*1.)*(uv1.x);


//shadow
float nolight = (1.-rtx);
highp float shadow = smoothstep(0.87,0.82,uv1.y);
shadow *= nolight;
if(detect_ore==false){
shadow = mix(0.0,0.3,shadow);
diffuse.rgb = mix(diffuse.rgb,vec3(0.,0.,0.2),shadow);}

//render
highp float render = smoothstep(0.5,0.2,color.g);
render *= nolight;
render = mix(0.0,0.3,render);
diffuse.rgb = mix(diffuse.rgb,vec3(0.,0.,0.2),render*1.0-shadow*rainy);


//activate tonemapping
diffuse.rgb = Tm(diffuse.rgb);



//detect
float detect = texture2D(TEXTURE_1, vec2(0., 1.)).r;
detect = clamp((detect-.5)*2., 0., 1.);
float detect_days = detect*uv1.y;
float detect_night = 1.-detect*uv1.y;
float detect_dusk = .5-abs(.5-detect)*uv1.y;
bool ISwater = (color.b>color.r);


//direct light
highp vec3 dir = -normalize(cross(dFdx(w_pos),dFdy(w_pos)));
float heh = 1.0-uv1.x;
float hih = uv1.y;
float hoh = 1.0-rainy;
float huh = 1.0-nighty;
float side_y = max(0.0,dir.y);
float side_yy = max(0.0,-dir.y);
float side_zz = max(dir.z,-dir.z);
float side_z = max(dir.z,-dir.z);
float side_x = max(dir.x,-dir.x);
if(detect_ore==false){
//diffuse.rgb = mix(diffuse.rgb, diffuse.rgb*0.2, side_y*hih*heh*hoh);
diffuse.rgb *= mix(vec3(0.95), vec3(0.3,0.3,0.5), side_z*hih*heh*hoh);
diffuse.rgb *= mix(vec3(0.95), vec3(0.4), side_y*hih*heh*hoh);
diffuse.rgb *= mix(1.0,1.2,side_x*hih*heh*hoh);}




if(fract(position.y)==0.0 || fract(position.y)== 0.5){


}

#ifdef BLEND
if(color.b>color.g||color.b>color.r){
highp float weww =
cos(noise(vec2(TIME*1.2+position.z+position.x+position.x+position.z*1.2,TIME*1.2+position.x+position.z+position.x+position.x*1.4+position.z*1.)))+cos(noise(vec2(TIME*1.2+position.z+position.x+position.x+position.x*1.2,TIME*1.2+position.x+position.z+position.x+position.x*1.4+position.z*1.)));

//diffuse.rgb *= vec3(0.5,1.1,1.3);
//highp float awan = clouds(TIME*-0.02+wpos.xz/50.);
highp float awan = clouds(TIME*-0.005+wpos.xz/wpos.y/-5.*abs(weww))*uv1.y;
//diffuse.rgb += noise(wpos.xyz*.5*abs(weww))*uv1.y;

if(FOG_COLOR.r *1.0>FOG_COLOR.b){//sunset
diffuse.rgb += awan*weww*vec3(1.3,0.4,0.1);

}

if(FOG_COLOR.r < 0.15 && FOG_COLOR.g < 0.15){//night
diffuse.rgb += awan*weww*vec3(0.4);
}

if(FOG_CONTROL.x < 0.55 && FOG_CONTROL.x > 0.1){
diffuse.rgb += awan*weww*vec3(0.5);
}

diffuse.rgb += awan*weww*vec3(1.0);

diffuse.rgb *= mix(vec3(0.9),vec3(0.8),abs(weww))*uv1.y;

/*
highp vec4 specular_albedo = vec4(0.8, 0.8, 0.8, 2.0);
highp float specular_power = 130.0;
highp vec3 light_pos = vec3(1000., 600., 0.);
highp vec3 P = wpos;
highp vec3 N = normalize(wpos);
highp vec3 V = normalize(-wpos);
highp vec3 L = normalize(light_pos - P);
highp vec3 R = reflect(-L, N);
highp vec4 specular =
        pow( max(0.0, dot(R, V)),
            specular_power ) *
        specular_albedo;
diffuse += specular;
*/


//clamp(length(wpos.xz)/RENDER_DISTANCE*2.0,0.0,1.0)*vec4(0.4,0.4,0.4,0.4);

if(FOG_COLOR.r *1.0>FOG_COLOR.b){//sunset
highp float su = sunny(wpos.zy/wpos.x/.5/vec2(1.3,7.)*abs(weww), 0.02, 0.3);
highp float su2 = sunny(wpos.zy/wpos.x/.5/vec2(1.3,7.)*abs(weww), 1., 0.7);
//diffuse.rgb *= vec3(1.0,0.9,0.5);
diffuse.rgb += su2*weww*vec3(1.0);
diffuse.rgb += su*weww*vec3(1.5);
}

highp float su = sunny(p.xz*7.5, 0.02, 0.1);
highp float su2 = sunny(p.xz, 1., 0.4);

float brr = 1.0-awan;
/*
if(-dir.y > .0){
float wetnis = clamp(length(-wpos/wpos.y*3.0)/RENDER_DISTANCE * 7.0, 0.1, 0.5) * 0.92;
diffuse += vec4(1.0)*wetnis*brr*uv1.y;}
*/
}

#endif



if(!(color.b > color.r)){
if(isWater==true){
diffuse.rgb *= vec3(.5,1.1,1.3);

}}
if(water==true){
	//diffuse.rgb *= vec3(1.);
	diffuse.a *= 0.6;
}

#ifdef FOG
if(FOG_CONTROL.x == 0.0 && FOG_COLOR.b > FOG_COLOR.r){
highp float wew = cos(noise(vec2(TIME*1.2+position.z+position.x+position.x+position.z*1.2,TIME*1.2+position.x+position.z+position.x+position.x*1.4+position.z*1.)));

diffuse.rgb *= vec3(.5,1.1,1.3);
//diffuse.rgb *= clamp(abs(wew)vec3(1.),0.0,2.);
diffuse.rgb *= mix(vec3(2.5),vec3(.9),abs(wew));

	}

#endif

/*Player Shadow
highp vec3 shadowPos = wpos;
shadowPos.z *= 0.5;
shadowPos.x *= 1.0;
shadowPos.x += 0.5;
shadowPos.z -= 0.5;
highp float p_shadow = max(player(shadowPos.xz), 0.0);
vec3 s_color = vec3(0.0);

diffuse.rgb *= mix(vec3(1.0), s_color, p_shadow);*/


#ifndef ALPHA_TEST
if(-dir.y > .0){
float wetnes = clamp(length(-wpos/wpos.y*6.0)/RENDER_DISTANCE * 2.0, 0., 1.) * 0.92;
diffuse += FOG_COLOR*vec4(0.9)*wetnes*side_yy*uv1.y*0.57*rainy;}
#endif

//rain effect
if(-dir.y > .0){
float AA = noise(position.xz)*0.090;
float BB = clamp(length(wpos.xyz)/RENDER_DISTANCE*0.3,0.0,0.9);
diffuse.rgb += AA*rainy*uv1.y;
diffuse.rgb += BB*rainy;
}


vec3 black = vec3(0,0,0)/255.;

if(detect_ore){
diffuse.rgb += 0.1;}


//fog
vec3 fog = mix(diffuse.rgb, FOG_COLOR.rgb*vec3(1.,1.15,1.3), clamp(length(-wpos)/RENDER_DISTANCE*1.5,0.0,0.6));
diffuse.rgb = fog;

/*
//fog v2
float foggy = clamp(length(-wpos)/RENDER_DISTANCE*1.,.0,0.6);
vec3 fogs = mix(diffuse.rgb,FOG_COLOR.rgb*vec3(1.,1.15,1.3),foggy);
diffuse.rgb = fogs;
*/


//rain fog
if(nether==false){
float rfoggy = clamp(length(-wpos)/RENDER_DISTANCE*7.0,0.1,0.6);
vec3 rfogs = mix(diffuse.rgb,mix(diffuse.rgb,vec3(0.6),rfoggy),rainy);
diffuse.rgb = rfogs;
}

//nether fog
if(nether==true){
float nfoggy = clamp(length(-wpos)/RENDER_DISTANCE*1.0,0.1,0.5);
vec3 nfogs = mix(diffuse.rgb,mix(diffuse.rgb,FOG_COLOR.rgb,nfoggy),rainy);
diffuse.rgb = nfogs;
}



#ifdef FOG
	diffuse.rgb = mix( diffuse.rgb, fogColor.rgb, fogColor.a );
#endif


	gl_FragColor = diffuse;
	
#endif // BYPASS_PIXEL_SHADER
}